/*
 * $Header: /home/cvs/jakarta-tomcat/proposals/catalina/src/share/org/apache/tomcat/Interceptor.java,v 1.4 2000/01/24 08:31:30 craigmcc Exp $
 * $Revision: 1.4 $
 * $Date: 2000/01/24 08:31:30 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 


package org.apache.tomcat;


import java.io.IOException;
import javax.servlet.ServletException;


/**
 * An <b>Interceptor</b>, as the name implies, intercepts request processing
 * prior to, and after, the call to a Container's <code>service()</code>
 * method.  Note that Interceptors may be associated with <b>any</b> type of
 * Container, so it is up to the Interceptor implementation to avoid
 * class cast exceptions.  An Interceptor can refuse to be associated with
 * a particular Container by returning <code>IllegalArgumentException</code>
 * to the <code>setContainer()</code> method.
 * <p>
 * The Container with which an Interceptor is associated must guarantee that
 * the <code>postService()</code> method will be called if the
 * <code>preService()</code> method was called, even in the face of exceptions.
 *
 * @author Craig R. McClanahan
 * @version $Revision: 1.4 $ $Date: 2000/01/24 08:31:30 $
 */

public interface Interceptor {


    // ------------------------------------------------------------- Properties


    /**
     * Return the Container with which this Interceptor is associated, if any.
     */
    public Container getContainer();


    /**
     * Set the Container with which this Interceptor is associated.
     *
     * @param container The newly associated Container
     *
     * @exception IllegalArgumentException if this Interceptor refuses to be
     *  associated with the specified Container
     * @exception IllegalStateException if this Interceptor is already
     *  associated with a different Container.
     */
    public void setContainer(Container container);


    /**
     * Return descriptive information about this Interceptor implementation and
     * the corresponding version number, in the format
     * <code>&lt;description&gt;/&lt;version&gt;</code>.
     */
    public String getInfo();


    // --------------------------------------------------------- Public Methods


    /**
     * Perform pre-processing for this request.  This method will be invoked
     * prior to any call to the underlying Container's <code>service()</code>
     * method, which will be followed by a call to this Interceptor's
     * <code>postService()</code> method.
     * <p>
     * A <code>preService()</code> method has the following choices:
     * <ul>
     * <li>Do nothing, and return <code>true</code>  This would be normal for
     *     an Interceptor that only performs post-processing of a request.
     * <li>Examine and/or modify the properties of the specified Request
     *     and/or Response, and return <code>true</code>.  This would be normal
     *     for an Interceptor that performs pre-processing, but does not
     *     complete the response.
     * <li>Examine the properties of the specified Request, completely generate
     *     the corresponding Response, and return <code>false</code>.  This
     *     would be normal for an Interceptor that decides to pre-empt the
     *     normal processing cycle for this request, such as a security
     *     implementation that needs to challenge the user for credentials.
     * <li>Throw an <code>IOException</code> if an input/output error is
     *     encountered during processing.
     * <li>Throw a <code>ServletException</code> if an unacceptable condition
     *     is encountered.  <b>DESIGN WARNING</b>:  Because
     *     <code>ServletException</code> is reserved for reporting application
     *     level errors from servlets, this action is considered bad form.
     * </ul>
     *
     * @param request Request to be processed
     * @param response Response to be produced, if desired
     *
     * @return <code>true</code> if processing of this request and response
     *  should continue, or <code>false</code> if creation of this response
     *  has been completed
     *
     * @exception IOException if an input/output error occurred while
     *  processing this request
     * @exception ServletException if a ServletException was thrown
     *  while processing this request
     */
    public boolean preService(Request request, Response response)
	throws IOException, ServletException;


    /**
     * Perform post-processing for this request.  This method <b>MUST</b>
     * be called, with the same request and response arguments, if the
     * <code>preService()</code> method was called for this request.
     * <p>
     * The <code>postService()</code> method may examine, but not modify, the
     * properties of the specified Request and Response.
     *
     * @param request Request that was processed
     * @param response Response that was produced
     * @param throwable Exception thrown by the Container to which this
     *  Interceptor is attached, if any; otherwise <code>null</code>
     */
    public void postService(Request request, Response response,
			    Throwable throwable);


}
